#! /bin/env python
""" Calculate the attribute sizes for a given directory.
"""

from BeOS.fsattr import *
import sys
import os

class AttrSize:
	def __init__( self ):
		self.size = 0
		self.num = 0
		self.files = 0
		self.total_files = 0
		self.largest = 0

	def Results( self ):
		if self.num > 0:
			print "Found %d attributes (total size of %d bytes) in %d files." % ( self.num, self.size, self.total_files )
			print "Average:", float( self.size ) / float( self.files ), "bytes per file over", self.files, "files."
			print float( self.files ) / float( self.total_files ) * 100.0, "% of files have attributes."
			print "Largest was %d bytes." % ( self.largest )
		else:
			print "Found 0 attributes in %d files." % ( self.total_files )

def visitor( obj, dir, names ):
	print "Visiting", dir, "..."
	for file in names:
		size, num = calc_attr_size( dir + "/" + file )
		if num > 0:
			obj.size = obj.size + size
			obj.num = obj.num + num
			obj.files = obj.files + 1
		if size > obj.largest:
			obj.largest = size
	obj.total_files = obj.total_files + len( names )

def calc_attr_size( filename ):
	""" Calculate the size of an file's attributes.
	
	Returns the ( size, # of attributes ).
	"""
	try:
		attrs = read_attrs( filename, ATTR_SYMLINK )
	except IOError, retval:
		print "Error reading attributes for %s: %s" % ( filename, retval[0] )
		return ( 0, 0 )
	
	# type codes
	attr_keys = attrs.keys()
	attr_size = len( attr_keys ) * 4
	
	for key in attr_keys:
		attr_size = attr_size + len( key )
		
		type, data = attrs[key]

		if type in ( B_BOOL_TYPE, B_INT8_TYPE, B_UINT8_TYPE ):
			attr_size = attr_size + 1
		elif type in ( B_INT16_TYPE, B_UINT16_TYPE ):
			attr_size = attr_size + 2
		elif type in ( B_INT32_TYPE, B_SIZE_T_TYPE, B_SSIZE_T_TYPE, B_UINT32_TYPE, B_TIME_TYPE, B_FLOAT_TYPE, B_REF_TYPE, B_RGB_COLOR_TYPE ):
			attr_size = attr_size + 4
		elif type in ( B_INT64_TYPE, B_OFF_T_TYPE, B_UINT64_TYPE, B_DOUBLE_TYPE ):
			attr_size = attr_size + 8
		elif type == B_POINT_TYPE:
			attr_size = attr_size + 2 * 4	# 2x float
		elif type == B_RECT_TYPE:
			attr_size = attr_size + 4 * 4	# 4x float
		else:
			attr_size = attr_size + len( data )

	return ( attr_size, len( attr_keys ) )

def main():
	attr_sizer = AttrSize()

	for path in sys.argv[1:]:
		os.path.walk( path, visitor, attr_sizer )

	attr_sizer.Results()

if __name__ == "__main__":
	main()
